@@ -15,18 +15,22 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-#
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
+from collections import OrderedDict
 import re
 import string
 
-from collections import OrderedDict
-
-from checkbox_support.lib.bit import get_bitmask, test_bit
+from checkbox_support.lib.bit import get_bitmask
+from checkbox_support.lib.bit import test_bit
 from checkbox_support.lib.input import Input
 from checkbox_support.lib.pci import Pci
 from checkbox_support.lib.usb import Usb
 
-
 PCI_RE = re.compile(
     r"^pci:"
     r"v(?P<vendor_id>[%(hex)s]{8})"
@@ -77,13 +81,13 @@ OPENFIRMWARE_RE = re.compile(
     r"N(?P<name>.*?)"
     r"T(?P<type>.*?)"
     r"C(?P<compatible>.*?)")
-CARD_READER_RE = re.compile(r"SD|MMC|CF|MS|SM|xD|Card", re.I)
+CARD_READER_RE = re.compile(r"SD|MMC|CF|MS(?!ata)|SM|xD|Card", re.I)
 GENERIC_RE = re.compile(r"Generic", re.I)
 FLASH_RE = re.compile(r"Flash", re.I)
 FLASH_DISK_RE = re.compile(r"Mass|Storage|Disk", re.I)
 
 
-class UdevadmDevice:
+class UdevadmDevice(object):
     __slots__ = (
         "_environment",
         "_bits",
@@ -148,7 +152,7 @@ class UdevadmDevice:
                     return "WIRELESS"
             # Ralink wireless
             if "INTERFACE" in self._environment:
-                if (
+                if (self.driver and
                     self.driver.startswith('rt') and
                     self._environment["INTERFACE"].startswith('ra')
                 ):
@@ -175,7 +179,19 @@ class UdevadmDevice:
                 else:
                     return "NETWORK"
             if class_id == Pci.BASE_CLASS_DISPLAY:
-                if subclass_id == Pci.CLASS_DISPLAY_VGA:
+                # Not all DISPLAY devices are display adapters. The ones with
+                # subclass OTHER are usually uninteresting devices. As an
+                # exception, some vendors have recently begun to use the
+                # 0x80 (Pci.CLASS_DISPLAY_OTHER) subclass identifier. In order
+                # to correctly identify them special heuristics are needed,
+                # these are encapsulated in the known_to_be_video_device method
+                # (further below).
+                if (subclass_id == Pci.CLASS_DISPLAY_VGA or
+                    subclass_id == Pci.CLASS_DISPLAY_3D or
+                    (subclass_id == Pci.CLASS_DISPLAY_OTHER
+                     and known_to_be_video_device(
+                         self.vendor_id, self.product_id, class_id,
+                         subclass_id))):
                     return "VIDEO"
             if class_id == Pci.BASE_CLASS_SERIAL \
                and subclass_id == Pci.CLASS_SERIAL_USB:
@@ -606,7 +622,7 @@ class UdevadmDevice:
         return {a: getattr(self, a) for a in attributes if getattr(self, a)}
 
 
-class UdevadmParser:
+class UdevadmParser(object):
     """Parser for the udevadm command."""
 
     device_factory = UdevadmDevice
@@ -621,6 +637,12 @@ class UdevadmParser:
         if not device.bus:
             return True
 
+        # Do not ignore devices with bus == net and ID_NET_NAME_MAC
+        # These can be virtual network interfaces which don't have PCI
+        # product/vendor ID, yet still constitute valid ethX interfaces.
+        if device.bus == "net" and "ID_NET_NAME_MAC" in device._environment:
+            return False
+
         # Ignore devices without product AND vendor information
         if (device.product is None and device.product_id is None and
                 device.vendor is None and device.vendor_id is None):
@@ -650,7 +672,7 @@ class UdevadmParser:
         multi_pattern = re.compile(r"(?P<key>[^=]+)=(?P<value>.*)")
 
         stack = []
-        if isinstance(self.stream_or_string, str):
+        if isinstance(self.stream_or_string, type("")):
             output = self.stream_or_string
         else:
             output = self.stream_or_string.read()
@@ -760,7 +782,28 @@ def decode_id(id):
     return decoded_id.strip()
 
 
-class UdevResult:
+def known_to_be_video_device(vendor_id, product_id, pci_class, pci_subclass):
+    # Usually a video device has a PCI subclass_id of Pci.CLASS_DISPLAY_VGA or
+    # Pci.CLASS_DISPLAY_3d. However, some manufacturers which hadn't previously
+    # used a subclass_id of Pci.CLASS_DISPLAY_OTHER (0x80 or decimal 128) have
+    # begun to do so (as of 2013-2014); and others which previously used this
+    # class for uninteresting devices are now using it for actual video
+    # devices. This method encapsulates heuristics to decide if a device is a
+    # valid video adapter, based on product/vendor and pci class/subclass
+    # information.
+    if vendor_id == Pci.VENDOR_ID_AMD:
+        # AMD hadn't used subclass OTHER before, so all AMD devices we get
+        # asked about are VIDEO.
+        return True
+    if vendor_id == Pci.VENDOR_ID_INTEL:
+        # Intel recently (2014) started using subclass OTHER erratically, some
+        # older GPUs have subdevices with OTHER which are uninteresting. If
+        # Intel, we only consider OTHER devices as VIDEO if they are in this
+        # explicit list
+        return product_id in [0x0152]
+
+
+class UdevResult(object):
     def __init__(self):
         self.devices = {"device_list": []}
 
